
class GPU_Worker():
    def __init__(self, gpus = [], waitTimeInit = 30, waitTime = 60,
                 checkType = 0, reversed=False, p = True):
        self.activate  = False
        self.gpus      = gpus
        self.checkType = checkType
        self.waitTimeInit  = waitTimeInit
        self.waitTime  = waitTime
        self.reversed  = reversed
        self.availGPUs = []
        self.p = p

    def setGPU(self):
        if self.activate: time.sleep(self.waitTimeInit)
        else: self.activate = True



        availGPUs, count = [], 0
        pynvml.nvmlInit()
        while True:
            count += 1
            for gpu in self.gpus:
                handle = pynvml.nvmlDeviceGetHandleByIndex(gpu)

                # 1. 아무것도 돌지 않는 경우
                if self.checkType == 0 and len(pynvml.nvmlDeviceGetComputeRunningProcesses(handle)) == 0:
                    availGPUs.append(gpu)

                # # 2. 70% 이하를 사용하는 경우
                # elif self.checkType == 1 and self.getFreeRatio(gpu) < 70:
                #     availGPUs.append(gpu)


            # for proc in pynvml.nvmlDeviceGetComputeRunningProcesses(handle):
            #     result[gpu] = [proc.pid, proc.usedGpuMemory]

            if len(availGPUs) == 0:
                if self.p: print("{} : Wait for finish".format(count))
                time.sleep(self.waitTime)

            else:
                break
        self.availGPUs = availGPUs
        if self.p: print("Activate GPUS : ", self.availGPUs)

    def getGPU(self):
        if len(self.availGPUs) == 0: self.setGPU()
        return self.availGPUs.pop() if self.reversed else self.availGPUs.pop(0)





def Process_Worker(processes, gpuWorker, p = True):
    start = time.localtime()
    print("------ Start Running!! : {} ------".format(time2str(start)))

    for i, process in enumerate(tqdm(processes)):
        gpu = gpuWorker.getGPU()
        prefix = f"CUDA_VISIBLE_DEVICES={gpu} nohup "
        suffix = f" > {i+1}:gpu{gpu}.log 2>&1 &"
        if p:
            print("------ {}:GPU{}  {} ------".format(i + 1, gpu, process))
            # print("------ {}:GPU{} ------".format(i + 1, gpu))
            # print(process)
        subprocess.call(prefix + process + suffix, shell=True)

    end = time.localtime()
    print("------ End Running!! : {} ------".format(time2str(end)))
    print("Training Time :  : {} ------".format(time2str(end - start)))


# Wokring Sample
# gpuWorker = GPU_Worker(args.gpus, 30, 120)
# Process_Worker(processes, gpuWorker)


import torch
import pickle
def rangeBlock(block, vmin=0, vmax=5):
    loss = torch.arange(vmin, vmax, (vmax - vmin) / block, requires_grad=False).unsqueeze(dim=1)
    return loss



# DataFrame
def columnRename(df, ns):
    for n in ns:
        if n[0] in df.columns:
            df.rename(columns = {n[0]: n[1]}, inplace = True)
#     columnRemove(df, ['CON밸브A','CON밸브A연산값','CON밸브B','CON밸브개도','CON밸브연산값'])


def columnRemove(df, ns):
    delList = []
    for n in ns:
        if n in df.columns:
            delList.append(n)
    df.drop(delList, axis=1, inplace=True)
#     columnRename(df, [
#                 ['CON레벨센서', 'CON레벨A']
#                 , ['CON레벨센서2', 'CON레벨B'])



from fvcore.nn import FlopCountAnalysis, flop_count_table, flop_count_str
from thop import profile
import torch

class AverageMeter:
    ''' Computes and stores the average and current value. '''
    def __init__(self) -> None:
        self.reset()

    def reset(self) -> None:
        self.val = 0.0
        self.avg = 0.0
        self.sum = 0.0
        self.count = 0

    def update(self, val: float, n: int = 1) -> None:
        self.val = val
        self.sum += val * n
        self.count += n
        self.avg = self.sum / self.count
    def __str__(self):
        return "\
        end = time.time() \n\
        batch_time = AverageMeter() \n\
        batch_time.update(time.time() - end) \n\
        end = time.time() \n\
        avg_score = AverageMeter()\n\
        accuracy = 0.1\n\
        avg_score.update(accuracy)\n\
        losses = AverageMeter()\n\
        loss = 0\n\
        batch_size = 128\n\
        losses.update(loss.data.item(), batch_size)\n\
        print(f'time {batch_time.val:.3f} ({batch_time.avg:.3f})\t'\n\
              f'loss {losses.val:.4f} ({losses.avg:.4f})\t' \n\
              f'acc {avg_score.val:.4f} ({avg_score.avg:.4f})')"



def thop(model, size):
    input = torch.randn(size)
    macs, params = profile(model, inputs=(input,))
    return macs, params

def numel(model):
    return sum([m.numel() for m in model.parameters()])

def flops(model, inputs, p=1):
    flops = FlopCountAnalysis(model, inputs)
    if p == 1:
        print(flop_count_table(flops))
    elif p == 2:
        print(flop_count_str(flops))
    return flops.total(), flops.by_operator(), flops.by_module(), flops.by_module_and_operator()





import setuptools

setuptools.setup(
    name="joonmyung",
    version="1.2.0",
    author="JoonMyung Choi",
    author_email="pizard@korea.ac.kr",
    description="JoonMyung's Library",
    url="https://github.com/pizard/JoonMyung.git",
    license="MIT",
    py_modules=['utils'],
    packages=setuptools.find_packages(),
    zip_safe=False,
    install_requires=[

    ]
)

# git add .
# git commit
# git push

# python setup.py sdist
# python -m twine upload dist/*     # Remove All File
# ID:JoonmyungChoi



import os
import random

import torch
import numpy as np

def to_np(v):
    return v.detach().cpu().numpy()

def str2list(s):
    v = ast.literal_eval(s)
    if type(v) is not list:
        raise argparse.ArgumentTypeError("Argument \"%s\" is not a list" % (s))
    return v

def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

def time2str(time, type = 0):
    if type == 0:
        return "{:4d}.{:2d}.{:2d} {:2d}:{:2d}:{:2d}".format(time.tm_year, time.tm_mon, time.tm_mday, time.tm_hour, time.tm_min, time.tm_sec)
    else:
        raise ValueError()